from django.contrib.sitemaps.views import sitemap
from django.urls import include, re_path

from cms.sitemaps import CMSSitemap

from .views import check_uninstall_ok


urlpatterns = [
    re_path(r'^admin/~cmscloud-api/check-uninstall/$', check_uninstall_ok, name='cms-check-uninstall'),
    re_path(r'^sitemap.xml$', sitemap, {'sitemaps': {'cmspages': CMSSitemap}}),
    re_path(r'^robots\.txt', include('robots.urls')),
]



import inspect
from importlib import import_module


def is_subclass(cls, base_class):
    """
    Returns True if cls is an actual class (type)
    and a subclass of base_cls
    """
    return inspect.isclass(cls) and issubclass(cls, base_class)


def get_classes_from_module(app, module_name, from_base_class=None):
    module_path = '%s.%s' % (app, module_name)
    module_object = import_module(module_path)

    if not module_object:
        raise StopIteration

    for cls_name in dir(module_object):
        cls = getattr(module_object, cls_name)

        if not inspect.isclass(cls):
            continue
        elif not from_base_class:
            yield cls
        elif is_subclass(cls, from_base_class):
            yield cls



import json

from django.http import HttpResponse, HttpResponseBadRequest

from cms.app_base import CMSApp
from cms.models import CMSPlugin, Page
from cms.plugin_base import CMSPluginBase

from .utils import get_classes_from_module


def check_uninstall_ok(request):
    """
    Returns HttpResponse("ok")
    if there's no pages with menus, plugins, apphooks
    from the apps being uninstalled.
    """
    apps = request.GET.get('apps', '').split(',')

    if apps == ['']:
        return HttpResponseBadRequest("no apps provided")

    page_lookup = Page.objects.filter
    plugin_lookup = CMSPlugin.objects.filter

    installed_apphooks = []
    installed_menus = []
    installed_plugins_by_type = {}

    for app in apps:
        # generator of CMSPluginBase subclasses
        # found in cms_plugins.py
        cms_plugins_classes = get_classes_from_module(
            app=app,
            module_name='cms_plugins',
            from_base_class=CMSPluginBase
        )

        for plugin_class in cms_plugins_classes:
            plugin_type = plugin_class.__name__

            count = plugin_lookup(plugin_type=plugin_type).count()

            if count:
                installed_plugins_by_type[plugin_type] = count

        # generator of CMSApp subclasses
        # found in cms_app.py
        cms_apphook_classes = get_classes_from_module(
            app=app,
            module_name="cms_apps",
            from_base_class=CMSApp
        )

        for apphook_class in cms_apphook_classes:
            hook = apphook_class.__name__

            if hook in installed_apphooks:
                continue
            elif page_lookup(application_urls=hook).exists():
                installed_apphooks.append(hook)

        # generator of classes found in menu.py module
        old_cms_menu_classes = set(get_classes_from_module(app=app, module_name="menu"))

        # 3.4 supports but does not require cms_menus module
        # will be required in 3.5
        new_cms_menu_classes = set(get_classes_from_module(app=app, module_name="cms_menus"))

        for menu_class in (old_cms_menu_classes | new_cms_menu_classes):
            menu = menu_class.__name__
            # Only look at menus that are "cms_enabled"
            cms_enabled = getattr(menu_class, 'cms_enabled', False)

            if menu in installed_menus or not cms_enabled:
                continue
            elif page_lookup(navigation_extenders=menu).exists():
                installed_menus.append(menu)

    if installed_plugins_by_type or installed_apphooks or installed_menus:
        result = {
            'plugins': installed_plugins_by_type,
            'apphooks': installed_apphooks,
            'menus': installed_menus
        }
    else:
        result = 'ok'
    return HttpResponse(json.dumps(result), content_type="application/json")



from django.urls import include, re_path


urlpatterns = [
    re_path(r'^api/~select2/', include('django_select2.urls')),

    # required by aldryn-forms
    re_path(r'^api/~captcha/', include('captcha.urls')),
]



#!/usr/bin/env python
from setuptools import find_packages, setup

from aldryn_django_cms import __version__


REQUIREMENTS = [
    "django-cms==4.0.1.dev7",
    "aldryn-addons",
    "requests",
    # NOTE: django-cms doesn't require this, but many of the addons do.
    #       If it is used, however, then it must be >=1.0.9 for CMS 3.3+.
    "django-treebeard>=4.0.1",  # django-cms
    "djangocms-admin-style",  # django-cms
    "django-select2>=6.2",
    # Other common
    # ------------
    # TODO: mostly to be split out into other packages
    "aldryn-snake",
    "django-compressor",
    "django-parler",
    # Django Sortedm2m 1.3 introduced a regression, that was fixed in 1.3.2
    # See https://github.com/gregmuellegger/django-sortedm2m/issues/80
    "django-sortedm2m>=1.5.0",
    "django-robots",
    "django-simple-captcha<=0.5.20",
    "lxml",
    "YURL",
]


CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Environment :: Web Environment",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: BSD License",
    "Operating System :: OS Independent",
    "Framework :: Django",
    "Framework :: Django :: 3.2",
    "Framework :: Django :: 4.2",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
    "Topic :: Internet :: WWW/HTTP",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    "Topic :: Software Development",
    "Topic :: Software Development :: Libraries",
]


setup(
    name="aldryn_django_cms",
    version=__version__,
    author="Divio AG",
    author_email="info@divio.ch",
    url="https://github.com/divio/aldryn-django-cms",
    license="BSD-3-Clause",
    description="An opinionated django CMS setup bundled as an Divio Cloud addon",
    long_description=open("README.rst").read(),
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIREMENTS,
    classifiers=CLASSIFIERS,
    test_suite="tests.settings.run",
)
